<?php

/**
 * @file
 * Definition of Drupal\mcapi\CurrencyFormController.
 */

namespace Drupal\mcapi;

use Drupal\Core\Entity\EntityFormController;
use Drupal\Component\Plugin\PluginManagerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

class CurrencyFormController extends EntityFormController {

  /**
   * The currency plugin manager
   *
   * @var \Drupal\Component\Plugin\PluginManagerBase
   */
  protected $pluginCurrencyManager;

  /**
   * The widget plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerBase
   */
  protected $pluginWidgetManager;

  /**
   * Constructs a new CurrencyFormController.
   *
   * @param \Drupal\Component\Plugin\PluginManagerBase $plugin_manager
   *   The widget or formatter plugin manager.
   */
  public function __construct(PluginManagerBase $currency_manager, PluginManagerBase $widget_manager) {
    $this->pluginCurrencyManager = $currency_manager;
    $this->pluginWidgetManager = $widget_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.mcapi.currency_type'),
      $container->get('plugin.manager.mcapi.currency_widget')
    );
  }

  public function getWidgetPlugin($configuration) {
    $plugin = NULL;

    if ($configuration) {
      $plugin = $this->pluginWidgetManager->getInstance(array(
        'currency' => $this->entity,
        'configuration' => $configuration
      ));
    }

    return $plugin;
  }

  /**
   * Get the widget options.
   */
  public function getWidgetOptions($currency_type) {
    return $this->pluginWidgetManager->getOptions($currency_type);
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::form().
   */
  public function form(array $form, array &$form_state) {
    $form = parent::form($form, $form_state);
    $currency = $this->entity;

    $form_state += array(
      'widget_settings_edit' => NULL,
      'widget_settings' => array(
        $currency->widget => $currency->widget_settings
      ),
    );

    $form['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name of currency'),
      '#description' => t('Use the plural'),
      '#default_value' => $currency->label(),
      '#size' => 40,
      '#maxlength' => 80,
      '#required' => TRUE,
      '#weight' => 0,
    );

    $form['id'] = array(
    	'#type' => 'machine_name',
    	'#default_value' => $currency->id(),
    	'#machine_name' => array(
    		'exists' => 'mcapi_currency_load',
    		'source' => array('name'),
    	),
    	'#disabled' => !$currency->isNew(),
      '#weight' => 1,
    );

    //get all users in the exchanges that this currency is in.
    //which exchanges reference this currency?
    $exchange_ids = db_select('mcapi_exchange__field_currencies', 'c')
      ->fields('c', array('entity_id'))
      ->condition('field_currencies_target_id', $this->entity->id())
      ->execute()->fetchCol();
    //now get all the members of those exchanges who have manage permission
    //would be great if we could somehow just feed into a entity_reference widget here
    $roles = user_roles(TRUE, 'manage own exchanges');
    $options = array(1 => user_load(1)->label());
    if ($roles) {
      $query = db_select('users', 'u')->fields('u', array('uid'));
      $query->join('users_roles', 'ur', 'ur.uid = u.uid');
      $query->join('user__field_exchanges', 'e', 'e.entity_id = u.uid');
      $uids = $query->condition('ur.rid', array_keys($roles))
        ->execute()->fetchCol();
      //wow this is getting long-winded
      foreach (entity_load_multiple('user', $uids) as $account) {
        $options[$account->id()] = $account->label();
      }
    }

    $form['uid'] = array(
    	'#title' => t('Comptroller'),
      '#description' => t('The one user who can edit this currency'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $currency->get('uid')->value
    );

    $form['acknowledgement'] = array(
      '#type' => 'container',
      '#children' => implode("\n<br /><br />\n", array(
        t('Acknowledgement currencies are abundant - they are issued whenever valued is created; they can be used as a medium of exchange but there is no guarantee of redemption.'),
        t("These are sometimes called 'social' currencies, because by encouraging and recognising volunteer service, they bind the community together."),
        t('This is the choice for all timebanking systems and most LETS.')
      )),
      '#states' => array(
        'visible' => array(
          ':input[name="issuance"]' => array('value' => CURRENCY_TYPE_ACKNOWLEDGEMENT)
        )
      ),
      '#weight' => 3,
    );
    $form['exchange'] = array(
      '#type' => 'container',
      '#children' => implode("\n<br /><br />\n", array(
        t("Exchange currencies are 'sufficient' - they are issued and redeemed as as users earn and spend."),
        t('The sum of all balances of active accounts, including the reservoir account, is zero, and ideally, accounts are returned to zero before being deactivated.'),
        t('To stop accounts straying too far from zero, positive and negative balance limits are often used.'),
        t('This model is sometimes called mutual credit, barter, or reciprocal exchange.'),
      )),
      '#states' => array(
        'visible' => array(
          ':input[name="issuance"]' => array('value' => CURRENCY_TYPE_EXCHANGE)
        )
      ),
      '#weight' => 3,
    );
    $form['commodity'] = array(
      '#type' => 'container',
      '#children' => implode("\n<br /><br />\n", array(
        t('Commodity currencies are scarce - the quantity is tied to the amount of a valuable commodity in a trusted warehouse.'),
        t('They are valued according to that commodity, and redeemed for that commodity, although fractional reserve rules may apply.'),
        t('Effectively the commodity is monetised, this brings confidence to the commodity, for the cost of the stuff in storage.'),
        t("This would be the choice for all 'dollar-backed' complementary currencies.")
      )),
      '#states' => array(
        'visible' => array(
          ':input[name="issuance"]' => array('value' => CURRENCY_TYPE_COMMODITY)
        )
      ),
      '#weight' => 3,
    );


    $form['issuance'] = array(
      '#title' => t('Basis of issuance'),
      '#description' => t('Currently only affects visualisation.'),
      '#type' => 'radios',
      '#options' => array(
        CURRENCY_TYPE_ACKNOWLEDGEMENT => t('Acknowledgement', array(), array('context' => 'currency-type')),
        CURRENCY_TYPE_EXCHANGE => t('Exchange', array(), array('context' => 'currency-type')),
        CURRENCY_TYPE_COMMODITY => t('Backed by a commodity', array(), array('context' => 'currency-type')),
      ),
      '#default_value' => property_exists($currency, 'issuance') ? $currency->issuance : 'acknowledgement',
      //this should have an API function to work with other transaction controllers
      //disable this if transactions have already happened
      '#disabled' => (bool)$this->entity->transactions(),
      '#weight' => 4,
    );
    $form['display'] = array(
      '#title' => t('Appearance'),
      '#type' => 'fieldset',
      '#collapsible' => TRUE,
      '#prefix' => '<div id="currency-display-wrapper">',
      '#suffix' => '</div>',
      '#weight' => 8,
    );

    $currency_type = $this->pluginCurrencyManager->createInstance($currency->type);
    $type_definition = $this->pluginCurrencyManager->getDefinition($currency->type);

    $form['display']['type'] = array(
      '#type' => 'markup',
      '#markup' => $this->t('@label - @description', array('@label' => $type_definition['label'], '@description' => $type_definition['description'])),
    );

    $form['display']['settings'] = array();
    $form['display']['settings'] = $currency_type->settingsForm($form['display']['settings'], $form_state, $currency);
    if ($form['display']['settings']) {
      $form['display']['settings']['#tree'] = TRUE;
    }

    $form['display']['widget'] = array(
      '#type' => 'select',
      '#title' => $this->t('Widget'),
      '#default_value' => $currency->widget,
      '#options' => $this->getWidgetOptions($currency->type),
      '#attributes' => array('class' => array('field-plugin-type')),
    );

    $form['display']['widget_settings'] = array(
      '#tree' => TRUE,
    );

    $form['display']['widget_summary'] = array();

    $widget_type = isset($form_state['input']['widget']) ? $form_state['input']['widget'] : $currency->widget;

    $options = array(
      'type' => $widget_type,
      'settings' => isset($form_state['widget_settings'][$widget_type]) ? $form_state['widget_settings'][$widget_type] : array(),
    );
    $widgetPlugin = $this->getWidgetPlugin($options);

    $base_button = array(
      '#submit' => array(array($this, 'multistepSubmit')),
      '#ajax' => array(
        'callback' => array($this, 'multistepAjax'),
        'wrapper' => 'currency-display-wrapper',
        'effect' => 'fade',
      ),
    );

    if ($form_state['widget_settings_edit']) {
      // We are currently editing this field's plugin settings. Display the
      // settings form and submit buttons.
      $field_row['plugin']['settings_edit_form'] = array();

      if ($widgetPlugin) {
        // Generate the settings form and allow other modules to alter it.
        $settings_form = $widgetPlugin->settingsForm($form, $form_state);
        $this->alterWidgetSettingsForm($settings_form, $widgetPlugin, $form, $form_state);

        if ($settings_form) {
          $form['display']['widget_settings'] = array(
            '#type' => 'container',
            '#attributes' => array('class' => array('widget-plugin-settings-edit-form')),
            'label' => array(
              '#markup' => $this->t('Plugin settings'),
            ),
            'widget_settings' => array('#tree' => TRUE) + $settings_form,
            'actions' => array(
              '#type' => 'actions',
              'save_settings' => $base_button + array(
                '#type' => 'submit',
                '#name' => 'widget_plugin_settings_update',
                '#value' => $this->t('Update'),
                '#op' => 'update',
                '#limit_validation_errors' => array(array('widget_settings')),
              ),
              'cancel_settings' => $base_button + array(
                '#type' => 'submit',
                '#name' => 'widget_plugin_settings_cancel',
                '#value' => $this->t('Cancel'),
                '#op' => 'cancel',
                // Do not check errors for the 'Cancel' button, but make sure we
                // get the value of the 'plugin type' select.
                '#limit_validation_errors' => array(),
              ),
            ),
          );
        }
      }
    }
    else {
      if ($widgetPlugin) {
        // Display a summary of the current plugin settings, and (if the
        // summary is not empty) a button to edit them.
        $summary = $widgetPlugin->settingsSummary();

        // Allow other modules to alter the summary.
        $this->alterWidgetSettingsSummary($summary, $widgetPlugin);
        $form['display']['widget_summary'] = array();

        if (!empty($summary)) {
          $form['display']['widget_summary']['summary'] = array(
            '#markup' => '<div class="widget-plugin-summary">' . implode('<br />', $summary) . '</div>',
          );
        }
        if ($widgetPlugin->getSettings()) {
          $form['display']['widget_summary']['edit'] = $base_button + array(
            '#type' => 'image_button',
            '#name' => 'widget_settings_edit',
            '#src' => 'core/misc/configure-dark.png',
            '#attributes' => array('class' => array('widget-plugin-settings-edit'), 'alt' => $this->t('Edit')),
            '#op' => 'edit',
            // Do not check errors for the 'Edit' button, but make sure we get
            // the value of the 'plugin type' select.
            '#limit_validation_errors' => array(),
            '#prefix' => '<div class="widget-plugin-settings-edit-wrapper">',
            '#suffix' => '</div>',
          );
        }
        if (!empty($form['display']['widget_summary'])) {
          $form['display']['widget_summary'] += array(
            '#prefix' => '<div class="currency-plugin-summary clearfix">',
            '#suffix' => '</div>',
          );
        }
      }
    }

    $form['display']['prefix'] = array(
      '#title' => t('Prefix'),
      '#type' => 'textfield',
      '#default_value' => $currency->prefix,
      '#max_length' => 6,
      '#size' => 6,
    );
    $form['display']['suffix'] = array(
      '#title' => t('Suffix'),
      '#type' => 'textfield',
      '#max_length' => 6,
      '#size' => 6,
      '#default_value' => $currency->suffix,
    );

    $serials = $this->entity->transactions(array('currcode' => $currency->id(), 'value' => 0));
    $form['display']['zero'] = array(
      '#title' => t('Zero value display'),
      '#description' => t('Use html.') .' ',
      '#type' => 'textfield',
      '#default_value' => $currency->zero,
      //this is required if any existing transactions have zero value
      '#required' => $serials
    );
    if ($form['display']['zero']['#required']) {
      $form['display']['zero']['#description'] = t("Zero transaction already exist so this field is required");
    }
    else {
      $form['display']['zero']['#description'] = t("Leave blank to disallow zero value transactions");
    }
    $form['display']['color'] = array(
    	'#title' => t('Colour'),
    	'#description' => t('Colour may be used in visualisations'),
    	'#type' => 'color',
    	'#default_value' => $currency->color,
//    	'#weight' =>
    );

    $form['additional_settings'] = array(
      '#type' => 'vertical_tabs',
      '#weight' => 15
    );
    $weight = 0;

    $form['access_undo'] = array(
      '#title' => t('Undo access'),
      '#description' => t('Determine who can undo transactions in each state.') .' '. t('Any of the checked conditions must return TRUE'),
      '#type' => 'details',
      '#group' => 'additional_settings',
      '#tree' => TRUE,
    );
    foreach (mcapi_get_states() as $state) {
      if ($state->value == TRANSACTION_STATE_UNDONE) continue;
      $form['access_undo'][$state->value] = array(
        '#title' => t(
          "Transactions in state '@state': !description",
          array('@state' => $state->label, '!description' => $state->description)
        ),
        '#type' => 'checkboxes',
        '#options' => transaction_access_plugins(TRUE),
        '#default_value' => $currency->access_undo[$state->value]
      );
    }

    //any operations declared not by mcapi.module
    $other_ops = array();
    foreach (transaction_operations() as $op => $plugin) {
      if (in_array($op, array('create', 'view', 'undo'))) {
        //these special operations have their own settings
        continue;
      }
      if ($settings = $plugin->access_form($currency, $op)) {
        $other_ops[$op] = $settings;
      }
    }
    if (count($other_ops)) {//show them all in one tab
      $form['access_operations'] = array(
        '#title' => t("Other operations"),
        '#type' => 'details',
        '#group' => 'additional_settings',
        '#tree' => TRUE,
      ) + $other_ops;
    }

    $form['refresh'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Refresh'),
      '#op' => 'refresh_display',
      '#limit_validation_errors' => array(),
      '#submit' => array(array($this, 'multistepSubmit')),
      '#ajax' => array(
        'callback' => array($this, 'multistepAjax'),
        'wrapper' => 'currency-display-wrapper',
        'effect' => 'fade',
        // The button stays hidden, so we hide the Ajax spinner too. Ad-hoc
        // spinners will be added manually by the client-side script.
        'progress' => 'none',
      ),
      '#attributes' => array('class' => array('visually-hidden'))
    );

    $form['#attached'] = array(
      'css' => array(
        drupal_get_path('module', 'mcapi') . '/css/admin_currency.css',
      ),
      'js' => array(
        drupal_get_path('module', 'mcapi') . '/js/admin_currency.js',
      ),
    );

    return $form;
  }

  /**
   * {@inherit}
   */
  protected function actions(array $form, array &$form_state) {
    $actions = parent::actions();
    $storage = \Drupal::EntityManager()->getStorageController('mcapi_currency');
    if (!$storage->deletable($this->entity)) {
      unset($actions['delete']);
    }
    return $actions;
  }

  public function multistepSubmit($form, &$form_state) {
    $trigger = $form_state['triggering_element'];
    $op = $trigger['#op'];

    switch ($op) {
      case 'edit':
        $form_state['widget_settings_edit'] = TRUE;
        break;

      case 'update':
        $form_state['widget_settings'][$form_state['input']['widget']] = $form_state['values']['widget_settings'];
        $form_state['widget_settings_edit'] = NULL;
        break;

      case 'cancel':
        $form_state['widget_settings_edit'] = NULL;
        break;

      case 'refresh_display':
        break;
    }

    $form_state['rebuild'] = TRUE;
  }

  public function multistepAjax($form, &$form_state) {
    return array('display' => $form['display']);
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::validate().
   */
  public function validate(array $form, array &$form_state) {
    //check that the reservoir account is allowed to use the currency
    $callback = 'in_'. strtok($form_state['values']['access']['membership'], ':');
    $arg = strtok(':');
    if (!function_exists($callback)) {
      form_set_error('reservoir', t('Invalid callback @callback.', array('@callback' => $callback)));
    }
    elseif (!$callback(array($arg), $form_state['values']['reservoir'])) {
      form_set_error('reservoir', t('Reservoir account does not have access to the currency!'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submit(array $form, array &$form_state) {
    $form_state['values']['widget_settings'] = isset($form_state['widget_settings'][$form_state['values']['widget']]) ? $form_state['widget_settings'][$form_state['values']['widget']] : array();

    parent::submit($form, $form_state);
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   */
  public function save(array $form, array &$form_state) {
    $currency = $this->entity;
    foreach ($currency->access_undo as $key => $values) {
      $currency->access_undo[$key] = array_filter($values);
    }

    if ($currency->widget == CURRENCY_WIDGET_SELECT) {
      foreach(explode("\n", $currency->select) as $line) {
        list($cent, $display) = explode('|', $line);
        //TODO: this will all be handled by the plugin.
        $currency->divisions[$cent] = trim($display);
      }
    }
    else {
      $currency->divisions = array();
    }

    $status = $currency->save();

    if ($status == SAVED_UPDATED) {
      drupal_set_message(t('Currency %label has been updated.', array('%label' => $currency->label())));
    }
    else {
      drupal_set_message(t('Currency %label has been added.', array('%label' => $currency->label())));
    }

    $form_state['redirect_route'] = array(
      'route_name' => 'mcapi.admin_currency_list'
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function alterWidgetSettingsForm(array &$settings_form, $plugin, array $form, array &$form_state) {
    $context = array(
      'widget' => $plugin,
      'form_mode' => 'full',
      'form' => $form,
    );
    \Drupal::moduleHandler()->alter('field_widget_settings_form', $settings_form, $form_state, $context);
  }

  /**
   * {@inheritdoc}
   */
  protected function alterWidgetSettingsSummary(array &$summary, $plugin) {
    $context = array(
      'widget' => $plugin,
      'field' => 'currency_type_' . $this->entity->type,
      'entity' => $this->entity,
    );
    \Drupal::moduleHandler()->alter('field_widget_settings_summary', $summary, $context);
  }
}
